#include "scene.h"
#include "v3.h"
#include "m33.h"
#include "ppc.h"
#include "tm.h"
#include <stdlib.h>

Scene *scene;

using namespace std;

#include <fstream>

#include <iostream>

Scene::Scene() {

	gui = new GUI();
	gui->show();

	int u0 = 20;
	int v0 = 20;
	int w = 640;
	int h = 480;

	fb = new FrameBuffer(u0, v0, w, h);
	fb->label("SW Framebuffer");
	fb->show();

	float hfov = 55.0f;
	ppc = new PPC(hfov, fb->w, fb->h);

	gui->uiw->position(u0, v0 + fb->h + 60);

	Render();

}

void Scene::Render() {

	fb->SetBGR(0xFFFFFFFF);
	fb->redraw();

}

void Scene::DBG() {


	{
		TM tm;
		tm.LoadBin("geometry/teapot57K.bin");
		ppc->C = V3(0.0f, 0.0f, 200.0f);
		fb->SetBGR(0xFFFFFFFF);
		fb->ClearZB(0.0f);
		tm.RenderWireframe(ppc, fb);
		fb->redraw();
		return;
	}

	{
		TM tm;
		tm.SetRectangle(V3(0.0f, 0.0f, -100.0f), 45.0f, 30.0f);
//		tm.RenderPoints(ppc, fb);
		int stepsN = 360;
		for (int stepsi = 0; stepsi < stepsN; stepsi++) {
			fb->SetBGR(0xFFFFFFFF);
			tm.RotateAboutArbitraryAxis(V3(0.0f, 0.0f, -100.0f), V3(0.0f, 1.0f, 0.0f),
				1.0f);
			tm.RenderWireframe(ppc, fb);
			fb->redraw();
			Fl::check();
		}
		return;

	}

	{

		V3 p0(-20.0f, 0.0f, -100.0f), pp;
		V3 p1(-20.0f, 0.0f, -1000.0f);
		int stepsN = 5;
		for (int stepi = 0; stepi < stepsN; stepi++) {
//			fb->SetBGR(0xFFFFFFFF);
			V3 p = p0 + (p1 - p0) * ((float)stepi / (float)(stepsN - 1));
			fb->Draw3DPoint(p, V3(1.0f, 0.0f, 0.0f), ppc, 7);
			fb->redraw();
			Fl::check();
		}
		return;

	}


	{

		V3 p0(20.1f, 40.0f, 0.0f);
		V3 p1(50.9f, 240.0f, 0.0f);
		V3 c0(1.0f, 0.0f, 0.0f);
		V3 c1(0.0f, 1.0f, 0.0f);
		fb->DrawSegment(p0, c0, p1, c1);
		fb->redraw();
		return;
	}


	{
		V3 v(1.0f, 0.0f, 0.0f);
		for (int i = 0; i < 1000; i++) {
			float scf = (float)i / 999.0f;
			v[1] = scf;
			unsigned int color = v.GetColor();
			V3 v1; v1.SetFromColor(color);
			cerr << v1 - v << "          \r";
			fb->SetBGR(color);
			fb->redraw();
			Fl::check();
		}
		cerr << endl;
		return;
	}

	{
		int u0 = 100;
		int v0 = 50;
		int u1 = 300;
		int v1 = 400;
		int r = 30;
		int stepsN = 100;
		for (int stepi = 0; stepi < stepsN; stepi++) {
			fb->SetBGR(0xFFFFFFFF);
//			fb->DrawRectangle(u0 + stepi, v0, u1 + stepi, v1, 0xFF0000FF);
			fb->DrawCircle(u0 + stepi, v0, r, 0xFF0000FF);
			fb->redraw();
			Fl::check();
		}
		return;
	}

	{
		M33 m;
		m[0] = V3(1.0f, 0.0f, 0.0f);
		m[1] = V3(0.0f, 1.0f, 0.0f);
		m[2] = V3(0.0f, 0.0f, 1.0f);
		M33 m1;
		m1[0] = V3(-1.0f, 2.0f, 3.0f);
		m1[1] = V3(-2.0f, 2.0f, 3.0f);
		m1[2] = V3(-3.0f, -2.0f, 5.0f);

		cerr << m1 << endl << m * m1 << endl << m1 * m << endl;
		return;

		cerr << m << endl;
		V3 v0(1.0f, -2.0f, 0.5f);
		cerr << m * v0;
		cerr << v0;

		return;

	}

	{
		V3 v0(1.0f, -2.0f, 0.5f);
		V3 v1(-1.0f, -4.0f, 3.0f);
		cerr << v0 << v1;
		cerr << "v0v1 = " << v0 * v1 << endl;
		return;
	}

	V3 v(1.0f, 3.0f, -1.0f);
	cerr << v[1] << endl;
	v[1] = 2.0f;
	cerr << v[1] << endl;
	return;

	int u0 = 100;
	int v0 = 50;
	int u1 = 300;
	int v1 = 400;

	fb->DrawRectangle(u0, v0, u1, v1, 0xFF0000FF);
	fb->redraw();

	fb->SaveAsTiff("mydbg/rr.tif");
	
	FrameBuffer *fb1 = new FrameBuffer(30 + fb->w, 30, fb->w, fb->h);
	fb1->LoadTiff("mydbg/rr.tif");
	fb1->label("Loaded image");
	fb1->show();

	return;

	for (int u = 0; u < fb->w; u++) {
		fb->Set(u, fb->h / 2, 0xFF000000);
	}
	fb->redraw();
	cerr << "INFO: pressed DBG" << endl;

}